__author__ = u'Atamert \xd6l\xe7gen'
__copyright__ = u'Copyright 2009, \xd6l\xe7gen Bili\u015fim'
__credits__ = [u'Atamert \xd6l\xe7gen', 'Daniel Watkins']


__license__ = 'BSD'
__version__ = '1.1'
__maintainer__ = u'Atamert \xd6l\xe7gen'
__email__ = 'muhuk@muhuk.com'
__status__ = 'Production'



import re
import unicodedata
from django import forms as django_forms
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.encoding import force_unicode


class FieldsetError(ValueError):
    pass


def slugify(value):
    # stolen from django.template.defaultfilters.slugify
    # only difference is this functions is substituting
    # spaces with underscores instead of hyphens
    value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore')
    value = unicode(re.sub('[^\w\s-]', '', value).strip().lower())
    return re.sub('[-\s]+', '_', value)


class Fieldset(object):
    "Simple iterable for holding fieldset information."

    def __init__(self, form, title=None, fields=(),
                 description=None, extra_content=None):
        self.form = form
        self.title = title
        self.fields = fields
        self.description = description
        self.extra_content = extra_content or {}

    def __iter__(self):
        "Iterates through fields in the fieldset."
        for field in self.fields:
            yield django_forms.forms.BoundField(self.form,
                                                self.form.fields[field],
                                                field)

    def _html_output(self,
                     fieldset_html,
                     title_html,
                     description_html,
                     normal_row,
                     error_row,
                     row_ender,
                     help_text_html,
                     errors_on_separate_row,
                     top_errors=None,
                     error_class=django_forms.util.ErrorList,
                     label_suffix=u':'):
        output, hidden_fields = [], []
        # top_errors is not supplied when the
        # fieldset is rendered individually
        if top_errors is None:
            top_errors = []
            top_errors_on_fieldset = True
        else:
            top_errors_on_fieldset = False
        for bf in self:
            # Escape and cache in local variable.
            bf_errors = error_class([escape(error) for error in bf.errors])
            if bf.is_hidden:
                if bf_errors:
                    top_errors.extend([u'(Hidden field %s) %s' %
                                       (bf.name, force_unicode(e))
                                       for e in bf_errors])
                hidden_fields.append(unicode(bf))
            else:
                if errors_on_separate_row and bf_errors:
                    output.append(error_row % force_unicode(bf_errors))
                if bf.label:
                    label = escape(force_unicode(bf.label))
                    # Only add the suffix if the label does not end in
                    # punctuation.
                    if label_suffix:
                        if label[-1] not in u':?.!':
                            label += label_suffix
                    label = bf.label_tag(label) or u''
                else:
                    label = u''
                if bf.field.help_text:
                    help_text = help_text_html % force_unicode(
                                                           bf.field.help_text)
                else:
                    help_text = u''
                output.append(normal_row % {'errors': force_unicode(bf_errors),
                                            'label': force_unicode(label),
                                            'field': unicode(bf),
                                            'help_text': help_text})
        if hidden_fields:
            # Insert any hidden fields in the last row.
            str_hidden = u''.join(hidden_fields)
            if output:
                last_row = output[-1]
                # Chop off the trailing row_ender (e.g. '</td></tr>')
                # and insert the hidden fields.
                output[-1] = last_row[:-len(row_ender)] + \
                                str_hidden + row_ender
            else:
                # If there aren't any rows in the output, just append
                # the hidden fields.
                output.append(str_hidden)
        # Render fieldset
        if self.title:
            title = title_html % escape(force_unicode(self.title))
        else:
            title = u''
        if self.description:
            description = description_html % force_unicode(self.description)
        else:
            description = u''
        if top_errors_on_fieldset and top_errors:
            output.insert(0, error_row % force_unicode(top_errors))
        return mark_safe(fieldset_html % {'title': title,
                                          'description': description,
                                          'fields': u'\n'.join(output)})

    def as_table(self):
        "Returns this fieldset's fields rendered as HTML <tr>s -- " \
        "excluding the <table></table>."
        return self._html_output(*self.form._tmpl_table)

    def as_ul(self):
        "Returns this fieldset's fields rendered as HTML <li>s -- " \
        "excluding the <ul></ul>."
        return self._html_output(*self.form._tmpl_ul)

    def as_p(self):
        "Returns this fieldset's fields rendered as HTML <p>s."
        return self._html_output(*self.form._tmpl_p)


class FieldsetMixin(object):
    _tmpl_table = (
        u'<tr><th colspan="2">%(title)s%(description)s</th></tr>%(fields)s',
        u'<h2>%s</h2>',
        u'<div class="description">%s</div>',
        u'<tr><th>%(label)s</th><td>%(errors)s' \
                                          u'%(field)s%(help_text)s</td></tr>',
        u'<tr><td colspan="2">%s</td></tr>',
        u'</td></tr>',
        u'<br />%s',
        False,
    )
    _tmpl_ul = (
        u'<li>%(title)s%(description)s<ul>%(fields)s</ul></li>',
        u'<h2>%s</h2>',
        u'<div class="description">%s</div>',
        u'<li>%(errors)s%(label)s %(field)s%(help_text)s</li>',
        u'<li>%s</li>',
        u'</li>',
        u' %s',
        False,
    )
    _tmpl_p = (
        u'<div>%(title)s%(description)s%(fields)s</div>',
        u'<h2>%s</h2>',
        u'<div class="description">%s</div>',
        u'<p>%(label)s %(field)s%(help_text)s</p>',
        u'%s',
        u'</p>',
        u' %s',
        True,
    )

    def _validate_fieldsets(self):
        valid = False
        fields_defined = sum((fset[1]['fields'] for fset in self.fieldsets),
                             ())
        fields_set = set(fields_defined)
        # Fieldsets are valid if:
        #  * Each field is defined in a Fieldset
        #  * Each field is defined only once.
        if len(fields_defined) == len(fields_set) and \
                                        set(self.fields.keys()) == fields_set:
            valid = True
        return valid

    def validate_fieldsets(self, force=False):
        "Return ``True`` if ``fieldsets`` is defined properly."
        if force or not hasattr(self, '__fieldsets_valid'):
            self.__fieldsets_valid = self._validate_fieldsets()
        if not self.__fieldsets_valid:
            raise FieldsetError('Fieldset definition for %s is invalid. ' \
                                'Each field must be defined in one and ' \
                                'only one Fieldset.' % self.__class__.__name__)
        else:
            return True

    def iter_fieldsets(self):
        "Iterates fieldsets."
        self.validate_fieldsets()
        for title, options in self.fieldsets:
            yield Fieldset(self, title, **options)

    @property
    def fieldset_dict(self):
        # No need to call validate_fieldsets() since we
        # are using iter_fieldsets.
        if not hasattr(self, '__fieldset_dict'):
            self.__fieldset_dict = dict((slugify(fset.title), fset) for \
                                                fset in self.iter_fieldsets())
        return self.__fieldset_dict

    def _html_fieldset_output(self,
                              fieldset_html,
                              title_html,
                              description_html,
                              normal_row,
                              error_row,
                              row_ender,
                              help_text_html,
                              errors_on_separate_row):
        "Helper function for outputting fieldsets as HTML. " \
        "Used by as_fieldset_table(), as_fieldset_ul(), as_fieldset_p()."

        # Errors that should be displayed above all fields.
        top_errors = self.non_field_errors()
        output = []
        for fieldset in self.iter_fieldsets():
            fieldset_output = fieldset._html_output(
                fieldset_html,
                title_html,
                description_html,
                normal_row,
                error_row,
                row_ender,
                help_text_html,
                errors_on_separate_row,
                top_errors,
                error_class=self.error_class,
                label_suffix=self.label_suffix)
            output.append(fieldset_output)
        if top_errors:
            output.insert(0, error_row % force_unicode(top_errors))
        return mark_safe(u'\n'.join(output))

    def as_fieldset_table(self):
        "Returns this form's fieldsets rendered as HTML <tr>s -- " \
        "excluding the <table></table>."
        return self._html_fieldset_output(*self._tmpl_table)

    def as_fieldset_ul(self):
        "Returns this form's fieldsets rendered as HTML <li>s -- " \
        "excluding the <ul></ul>."
        return self._html_fieldset_output(*self._tmpl_ul)

    def as_fieldset_p(self):
        "Returns this form's fieldsets rendered as HTML <p>s."
        return self._html_fieldset_output(*self._tmpl_p)



from django import template
from django.conf import settings


register = template.Library()


DEFAULT_FORM_TEMPLATE = getattr(settings,
                                'FORMFIELDSET_DEFAULT_FORM_TEMPLATE',
                                'formfieldset/form.html')


@register.filter
def renderform(form, template_name=None):
    """Render form.

       If optional argument ``template_name`` is given that template will be
       used for rendering. Otherwise ``formfieldset/form.html`` will be used.
    """
    tmpl = template.loader.get_template(template_name or DEFAULT_FORM_TEMPLATE)
    return tmpl.render(template.Context({'form': form}))



from distutils.core import setup
from formfieldset import __version__, __maintainer__, __email__


license_text = open('LICENSE.txt').read()
long_description = open('README.rst').read()


setup(
    name = 'django-formfieldset',
    version = __version__,
    url = 'http://github.com/muhuk/django-formfieldset/tree/master',
    author = __maintainer__,
    author_email = __email__,
    license = license_text,
    packages = ['formfieldset', 'formfieldset.templatetags'],
    package_data= {'friends': ['templates/formfieldset/*.html']},
    data_files=[('', ['LICENSE.txt', 'README.rst'])],
    description = 'Fieldset Rendering For Non-Admin Forms',
    long_description=long_description,
    classifiers = ['Development Status :: 5 - Production/Stable',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content']
)



from django.test import TestCase
from django import forms as django_forms
import forms


class FieldsetValidationTestCase(TestCase):
    def setUp(self):
        class BaseForm(django_forms.Form, forms.FieldsetMixin):
            test_field1 = django_forms.CharField()
            test_field2 = django_forms.CharField()
            test_field3 = django_forms.CharField()

        self.base_form = BaseForm

    def test_missing_fields(self):
        class TestForm(self.base_form):
            fieldsets = (
                (u'Fieldset1', {'fields': ('test_field1',)}),
                (u'Fieldset2', {'fields': ('test_field2',)}),
            )

        self.assertRaises(forms.FieldsetError, TestForm().validate_fieldsets)

    def test_duplicate_fields(self):
        class TestForm(self.base_form):
            fieldsets = (
                (u'Fieldset1', {'fields': ('test_field1', 'test_field3')}),
                (u'Fieldset2', {'fields': ('test_field2', 'test_field3')}),
            )

        self.assertRaises(forms.FieldsetError, TestForm().validate_fieldsets)

    def test_valid_fieldset(self):
        class TestForm(self.base_form):
            fieldsets = (
                (u'Fieldset1', {'fields': ('test_field1',)}),
                (u'Fieldset2', {'fields': ('test_field2', 'test_field3')}),
            )

        self.assertEqual(TestForm().validate_fieldsets(), True)


class FieldsetAccessTestCase(TestCase):
    def setUp(self):
        class BaseForm(django_forms.Form, forms.FieldsetMixin):
            test_field1 = django_forms.CharField()
            test_field2 = django_forms.CharField()
            test_field3 = django_forms.CharField()

            fieldsets = (
                (u'Fieldset1', {'fields': ('test_field1',)}),
                (u'Fieldset2', {'fields': ('test_field2', 'test_field3')}),
            )

        self.base_form = BaseForm

    def test_iter_fieldsets(self):
        fieldsets = [fset.title for fset in self.base_form().iter_fieldsets()]
        self.assertEqual(len(fieldsets), 2)
        self.assertEqual('Fieldset1' in fieldsets, True)
        self.assertEqual('Fieldset2' in fieldsets, True)

    def test_fieldset_dict(self):
        fset_dict = self.base_form().fieldset_dict
        self.assertEqual(len(fset_dict), 2)
        self.assertEqual('fieldset1' in fset_dict.keys(), True)
        self.assertEqual('fieldset2' in fset_dict.keys(), True)
        self.assertRaises(AttributeError,
                          self.base_form().__setattr__, 'fieldset_dict', {})


class FieldsetRenderTestCase(TestCase):
    def setUp(self):
        class TestForm(django_forms.Form, forms.FieldsetMixin):
            test_field1 = django_forms.CharField()
            test_field2 = django_forms.CharField()
            test_field3 = django_forms.CharField()
            test_field4 = django_forms.CharField(
                                      widget=django_forms.widgets.HiddenInput)

            fieldsets = (
                (u'Fieldset1', {
                    'description': u'Test Description<sup>1</sup>',
                    'fields': ('test_field1',),
                }),
                (u'Fieldset2', {
                    'fields': ('test_field2', 'test_field3', 'test_field4'),
                }),
            )

            def clean_test_field2(self):
                if self.cleaned_data['test_field2'] != u'':
                    raise django_forms.ValidationError(
                                    [u'Test Error - Field Level - 1',
                                     u'Test Error - Field Level - 2'])

            def clean_test_field4(self):
                raise django_forms.ValidationError(
                                                 u'Test Error - Hidden Field')

            def clean(self):
                raise django_forms.ValidationError(u'Test Error - Top Level')
        self.test_form = TestForm

    def test_fieldset_render(self):
        """Test if the form is being rendered at all and all the elements are
           in the result.
        """
        form = self.test_form(data={'test_field2': u'Test Value',
                                    'test_field4': u'Test Value'})
        self.assertEqual(form.is_valid(), False)
        for method in ('as_fieldset_table', 'as_fieldset_ul', 'as_fieldset_p'):
            rendered = getattr(form, method)()
            # Are all errors rendered somehow?
            self.assertTrue(u'Test Error - Top Level' in rendered)
            self.assertTrue(u'Test Error - Hidden Field' in rendered)
            self.assertTrue(u'Test Error - Field Level - 1' in rendered)
            self.assertTrue(u'Test Error - Field Level - 2' in rendered)
            # Are all fields present?
            for field in form.fields.keys():
                bf= django_forms.forms.BoundField(form,
                                                  form.fields[field],
                                                  field)
                self.assertTrue(unicode(bf) in rendered)
            # Check for fieldset titles & decriptions
            self.assertTrue(u'Fieldset1' in rendered)
            self.assertTrue(u'Fieldset2' in rendered)
            self.assertTrue(u'Test Description<sup>1</sup>' in rendered)

    def test_individual_fieldset_render(self):
        form = self.test_form(data={'test_field2': u'Test Value',
                                    'test_field4': u'Test Value'})
        fieldset1 = form.fieldset_dict['fieldset1']
        fieldset2 = form.fieldset_dict['fieldset2']
        self.assertEqual(form.is_valid(), False)
        for method in ('as_table', 'as_ul', 'as_p'):
            rendered1 = getattr(fieldset1, method)()
            rendered2 = getattr(fieldset2, method)()
            # Are all errors rendered somehow?
            self.assertTrue(u'Test Error - Field Level - 1' in rendered2)
            self.assertTrue(u'Test Error - Field Level - 2' in rendered2)
            self.assertTrue(u'Test Error - Hidden Field' in rendered2)
            # Are all fields present?
            for fieldset, rendered in ((fieldset1, rendered1),
                                       (fieldset2, rendered2)):
                for field in fieldset.fields:
                    bf= django_forms.forms.BoundField(form,
                                                      form.fields[field],
                                                      field)
                    self.assertTrue(unicode(bf) in rendered)
            # Check for fieldset titles & decriptions
            self.assertTrue(u'Fieldset1' in rendered1)
            self.assertTrue(u'Fieldset2' in rendered2)
            self.assertTrue(u'Test Description' in rendered1)
